/******************************************************************/
/* filename:vardummy.g                                            */
/* Description:Generating dummy observations                      */
/*                  for a Minnesota Prior                         */
/*              created: Feb 26, 2005                             */
/*               Taeyoung Doh                                     */
/******************************************************************/

/*Set the values for hyperparameters**/

tau=3;          /*overall tightness */ 
d=0.5;          /*scaling down the variance for the coefficients of a distant lag */ 
w=1;            /*number of observations used for obtaining the prior for the covariance matrix of error terms*/ 
lambda=5;       /*tuning parameter for coefficients for constant*/ 
mu=2;           /*tuning parameter for the covariance between coefficients*/ 


/******************************************************** 
** Import data
** series (nobs,2)
*/
#include loaddata.g

nobs  = rows(series);       /* number of observations */
YY    = series[1:nobs,.];  
 
/*dimension for dummy observations*/ 
nv=cols(series); /*number of variables*/ 
nobs=rows(series); 
nlag=4; /*lag length*/ 
T0=23;  /*pre-sample size*/ 

nd=1+(nlag+w+nv)*(nv);  /*number of total dummy observations*/ 
ydum=zeros(nd,nv); 
xdum=zeros(nd,1+nlag*nv); 

/*Obtain mean and standard deviation from pre-sample data*/ 
YY=YY[1:T0,.];  
ybar=meanc(YY); ybar=ybar'; 
sbar=stdc(YY); sbar=sbar';        
sprior=ybar|sbar; 

/*Generate Dummy Observations*/ 
mprior=tau|d|w|lambda|mu; 
{xdum,ydum,breakss}=varprior(nv,nlag,mprior,sprior); 
x0=xdum; 
y0=ydum;  

y1=series[T0+1:nobs,.]; 
x1=zeros(nobs-T0,nv*nlag); 
i = 1;
do until i > nlag;
   x1[.,(i-1)*nv+1:i*nv] = series[T0+1-i:nobs-i,.];
   i = i+1;
endo;

x1=x1~ones(nobs-T0,1); 

/***************************************************************
The procedure for generating dummy observations  

****************************************************************/ 
proc(3)=varprior(a,b,c,e); 
/*a:nv, b:nlag, c:hyperparameters, e:pre-sample moments */ 
local ta, dd, ww, lam, mm,xdu,ydu,breakss,dsize,j,sig,lammean,mumean; 
ta=c[1,.];  dd=c[2,.]; ww=c[3,.]; lam=c[4,.]; mm=c[5,.]; 
dsize=1+(b+ww+1)*a; 
breakss=zeros(5,1); 

ydu=zeros(dsize,a); 
xdu=zeros(dsize,1+a*b); 

/*Dummies for the coefficients of the first lag*/ 
sig=e[2,.]'; sig=diagrv(zeros(a,a),sig); 
ydu[1:a,.]=ta*sig; 
xdu[1:a,.]=(ta*sig)~zeros(a,(b-1)*a+1); 
breakss[1,.]=a; 

/*Dummies for the coefficients of the remaining lag*/ 
ydu[breakss[1,.]+1:a*b,.]=zeros((b-1)*breakss[1,.],a);
j=1;
do until j>b-1; 
 xdu[breakss[1,.]+(j-1)*a+1:breakss[1,.]+j*a,.]=zeros(a,j*a)~ta*sig*((j+1)^dd)~zeros(a,1+(b-1-j)*a);  
 j=j+1; 
endo; 
breakss[2,.]=breakss[1,.]+(b-1)*a; 

/*Dummies for the covariance matrix of error terms */ 

ydu[breakss[2,.]+1:breakss[2,.]+ww*a,.]=ones(ww,1).*.sig; 
breakss[3,.]=breakss[2,.]+ww*a; 

/*Dummies for the coefficients of the constant term*/ 

lammean=lam*e[1,.];
ydu[breakss[3,.]+1,.]=lammean; 
xdu[breakss[3,.]+1,.]=lammean~lammean~lammean~lammean~lam; 
breakss[4,.]=breakss[3,.]+1; 

/*Dummies for the covariance matrix of coefficients of different lags */ 

mumean=diagrv(zeros(a,a),(mm*e[1,.])');
ydu[breakss[4,.]+1:breakss[4,.]+a,.]=mumean; 
xdu[breakss[4,.]+1:breakss[4,.]+a,.]=mumean~mumean~mumean~mumean~zeros(a,1); 
breakss[5,.]=breakss[4,.]+a; 

retp(xdu,ydu,breakss); 
endp; 
