/* filename:   dsgepred.g
** description: The program converts the DSGE parameters
**              into impulse responses, etc
** created:     21/10/99
*/

/* new; */
closeall;
library  pgraph, user;
cls;

#include pathspec.g;
#include dsgespec.g; 
#include dsgemod.src;

/* Open files with Parameter Draws
*/
mhrun  = "0";

lpath  = parapath $+"\\mhrun" $+ mhrun;
lpara  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pa";
open fhpara = ^lpara for read;

/* Define some Constants
*/
nsim   = 10000;        /* block size */
nuse   = 100;          /* use every nuse observation */

/* Initialize Output files
*/
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir=chk_dir(opath); 
oir11 = opath $+ "\\"$+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "r11";
oir12 = opath $+ "\\"$+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "r12";

oir21 = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "r21";
oir22 = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "r22";

oir31 = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "r31";
oir32 = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "r32";

ospy1 = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "sy1";
ospy2 = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "sy2";

ospl1 = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "sl1";
ospl2 = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "sl2";

ovard = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "vd";

create fhir11 = ^oir11 with IR11, nirf, 8;
create fhir12 = ^oir12 with IR12, nirf, 8;

create fhir21 = ^oir21 with IR21, nirf, 8;
create fhir22 = ^oir22 with IR22, nirf, 8;

create fhir31 = ^oir31 with IR31, nirf, 8; 
create fhir32 = ^oir32 with IR32, nirf, 8; 

create fhspy1 = ^ospy1 with SPY1, nomega, 8;
create fhspy2 = ^ospy2 with SPY2, nomega, 8;

create fhspl1 = ^ospl1 with SPL1, nomega, 8;
create fhspl2 = ^ospl2 with SPL2, nomega, 8;

create fhvard = ^ovard with VARD, 4, 8;

/* Initialization
*/ 

eofloop = 0;
loopct  = 0;

do until eofloop; 

   /* Read blocks of size nsim
   */
   parasim = readr(fhpara, nsim );
   npara = cols(parasim);
   nread = rows(parasim);

   indseq = seqa(1,1,nread);
   indseq = indseq % nuse;
   indseq = indseq .NE 0;
 
   parasim = delif(parasim,indseq);
   nread = rows(parasim);

   invalidsim = zeros(nread,1);
      
   Asim11 = zeros(nread,nirf);
   Asim12 = zeros(nread,nirf);
 
   Asim21 = zeros(nread,nirf);
   Asim22 = zeros(nread,nirf);
 
   Asim31 = zeros(nread,nirf); 
   Asim32 = zeros(nread,nirf); 

   Ssimy1 = zeros(nread,nomega);
   Ssimy2 = zeros(nread,nomega);
 
   Ssiml1 = zeros(nread,nomega);
   Ssiml2 = zeros(nread,nomega);
 
   vardsim= zeros(nread,4);  
   corrsim= zeros(nread,1) ;

   j = 1;
   do until j > rows(parasim);

      /* For each element of the block, compute the desired statistic
      */
     {TT,TC,T0,RR,RC,retcode} = dsgesolv(parasim[j,.]',mspec);
   
/*adding measurement equations*/ 
  
 if (mspec==0) or (mspec==2);

TT=(TT ~zeros(rows(TT),1))|(zeros(1,cols(TT)+1)); 
T0=T0|(zeros(1,cols(T0)));  RR=RR|(zeros(1,cols(RR))); 
TC=TC|(zeros(1,1));
yyirf = zeros(nirf,3);

ZZ = zeros(3,cols(TT));

 TT[8,8]=1; T0[8,1]=1; 
  ZZ[1,1]=1; /*hour*/       
  ZZ[2,3]=1;  ZZ[2,8]=1;  /*consumption*/
  ZZ[3,4]=1;  ZZ[3,8]=1;  /*output */ 
else; 

TT=(TT ~zeros(rows(TT),2))|(zeros(2,cols(TT)+2)); 
T0=T0|(zeros(2,cols(T0)));  RR=RR|(zeros(2,cols(RR))); 
TC=TC|(zeros(2,1));
TC[8]=(1-parasim[j,8])*(parasim[j,7]); 
yyirf = zeros(nirf,3);

 ZZ = zeros(3,cols(TT));
 TT[7,7]=1; T0[7,1]=1; 
 TT[8,8]=1; T0[8,2]=1; 
  ZZ[1,1] = 1;  ZZ[1,8]=1; /* hour */
  ZZ[2,3] = 1;  ZZ[2,7]=1; ZZ[2,8]=1; /* consumption  */ 
   ZZ[3,4] = 1; ZZ[3,7]=1; ZZ[3,8]=1; /*output  */

endif;       

     if  (mspec==0) or (mspec==1); 
     sigt = parasim[j,npara-1];
     sigp = parasim[j,npara];
     else; 
     sigt=parasim[j,npara-2]; 
     sigp=parasim[j,npara-1];     
     endif; 
     nstate = rows(TT);

  
     /* compute impulse responses for technology, preference 
     */

     IRFdt = zeros(3,nirf);
     IRFdp = zeros(3,nirf); 
  
     /* Define initial state vector vector 
     */
     alpdt = zeros(nstate,1);
     alpdp = zeros(nstate,1);
  

     /* Shocks
     */
   
     v_dt   = (100 |0  )*sigt ~ zeros(2,nirf-1);
     v_dp   = (0   |100  )*sigp ~ zeros(2,nirf-1);

     /* Initialize Loop variable t 
     */
     t=1;

     DO UNTIL t > nirf;

      /* update the vector of state variables alpha
      */
      alpdt = TT*alpdt + T0*v_dt[.,t];
      irfdt[.,t] = ZZ*alpdt;

      alpdp = TT*alpdp + T0*v_dp[.,t];
      irfdp[.,t] = ZZ*alpdp;

      t=t+1;

     ENDO;
        
     
     
     /* Variance Decomposition via impulse responses 
     */
/*
     vardy2 = ( irfdt[1,.]*irfdt[1,.]' / ( irfdt[1,.]*irfdt[1,.]' + irfdp[1,.]*irfdp[1,.]' ) )~
                 ( irfdp[1,.]*irfdp[1,.]' / ( irfdt[1,.]*irfdt[1,.]' + irfdp[1,.]*irfdp[1,.]' ) ); 
         
    vardl2 = ( irfdt[2,.]*irfdt[2,.]' / ( irfdt[2,.]*irfdt[2,.]' + irfdp[2,.]*irfdp[2,.]' ) )~
                ( irfdp[2,.]*irfdp[2,.]' / ( irfdt[2,.]*irfdt[2,.]' + irfdp[2,.]*irfdp[2,.]' ) );

*/
     Asim11[j,.] = irfdt[1,1:nirf];
     Asim12[j,.] = irfdp[1,1:nirf];
   
     Asim21[j,.] = irfdt[2,1:nirf];
     Asim22[j,.] = irfdp[2,1:nirf];
 
     Asim31[j,.] = irfdt[3,1:nirf]; 
     Asim32[j,.] = irfdp[3,1:nirf]; 
   
   /* Conduct Spectral Analysis
     */

    /*composition of state vector 
   ** h(t)=1, y(t)=4  */
  

    if (mspec==1) or (mspec==3); 
     TT=TT[1:6,1:6]; 
     T0=T0[1:6,.]; 
     TT=TT~zeros(6,2); 
     TT=TT|zeros(2,8);
    
     T0=T0|zeros(2,2); 
     TT[7,4]=1; TT[8,1]=1; 
     ZZ=zeros(2,8); 
     ZZ[1,4]=1; ZZ[1,7]=-1; ZZ[1,8]=1; 
     ZZ[2,1]=1; ZZ[2,8]=1; 
   
   else; 
    TT=TT[1:6,1:6]; 
     T0=T0[1:6,.]; 
    TT=TT~zeros(6,1); 
     TT=TT|zeros(1,7);
     T0=T0|zeros(1,2); 
     TT[7,4]=1;
     ZZ=zeros(2,7); 
     ZZ[1,4]=1; ZZ[1,7]=-1; 
     ZZ[2,1]=1;
   endif;
     vardl = zeros(1,2);
     vardy = zeros(1,2);

     {spec,gam} = spectr(TT, T0, ZZ, (sigt|0),mspec);
     Ssimy1[j,.] = spec[1,.];
     Ssiml1[j,.] = spec[2,.];
     vardy[1,1] = gam[1,1];
     vardl[1,1] = gam[2,2];


     {spec,gam} = spectr(TT, T0, ZZ, (0|sigp),mspec );
     Ssimy2[j,.] = spec[1,.];
     Ssiml2[j,.] = spec[2,.];
     vardy[1,2] = gam[1,1];
     vardl[1,2] = gam[2,2];

     vardy = vardy/sumc(vardy');
     vardl = vardl/sumc(vardl');
     vardsim[j,.] = vardy ~ vardl;

     eofparasim:

     j = j+1;
  endo;

  
  /* Write Output
  */
  wr = writer(fhir11,Asim11);
  wr = writer(fhir12,Asim12);
 
  wr = writer(fhir21,Asim21);
  wr = writer(fhir22,Asim22);

  wr = writer(fhir31,Asim31); 
  wr = writer(fhir32,Asim32); 

  wr = writer(fhspy1,Ssimy1);
  wr = writer(fhspy2,Ssimy2);

  wr = writer(fhspl1,Ssiml1);
  wr = writer(fhspl2,Ssiml2);
  
  wr = writer(fhvard,vardsim);

  loopct = loopct + 1;
  
  "Loop     " loopct;
  pause(1e-5);
  eofloop = eof(fhpara);

endo;

closeall  fhir11, fhir12,
          fhir21, fhir22,
          fhir31,  fhir32,   
          fhspy1, fhspy2,
          fhspl1, fhspl2, 
          fhpara, fhvard;

clear Asim11, Asim12,
      Asim21, Asim22,
      Asim31, Asim32, 
      Ssimy1, Ssimy2,
      Ssiml1, Ssiml2,
      vardsim;


/* Procedures Start Here
*****************************/

proc (2) = spectr(TTT, RRR, ZZZ, sig,tspec);
/* Compute the Spectrum as a function of the
** system matrices
*/
local nstate, sigma, spec, i, S1, Pt, GAM,
      p, d, q, TTsvd, RRsvd, Ptsvd,TT,RR,ZZ;

nstate = rows(TTT);
sigma = eye(2) .* (sig[1]^2 | sig[2]^2 );
spec = zeros(2,nomega);

if (tspec==0) or (tspec==2); 
/*Expand the state vector by adding eps A(t) */ 
TT=TTT~zeros(7,1); TT=TT|zeros(1,8); 
RR=RRR|zeros(1,2); RR[8,1]=1; 
ZZ=ZZZ~(1|0); 
else; 
/*Expand the state vector by addling eps A(t) and eps B(t) */ 
TT=TTT~zeros(8,2); TT=TT|zeros(2,10); 
RR=RRR|eye(2);  
ZZ=ZZZ~ones(2,2); ZZ[2,9]=0; 
endif; 
nstate=rows(TT);

i = 1;
do until i > nomega;
   
   S1 = complex( eye(nstate) - TT*cos(omegagr[i]),   TT*sin(omegagr[i]) );  
   spec[.,i] = real( diag( ZZ* (inv(S1)*(RR*sigma*RR')*inv(S1)'/(2*pi) )* ZZ' ) );

   i = i+1;

endo;

i = rank(TT);
{p, d, q} = svd1(TT);
q = q';
p = p[.,1:i];
q = q[1:i,.];
d = d[1:i,1:i];

TTsvd = q*p*d;
rrsvd = q*RR;

Ptsvd = reshape(inv(eye(i^2) - (TTsvd.*.TTsvd))*vec(RRsvd*SIGMA*RRsvd'),i,i);
Pt    = RR*sigma*RR' + p*d*Ptsvd*d*p'; 
GAM   = ZZ*Pt*ZZ';

/*
Pt = reshape(inv(eye(nstate^2) - (TT.*.TT))*vec(RR*SIGMA*RR'),nstate,nstate);
GAM = ZZ*Pt*ZZ';
*/

retp(spec,gam);
endp;
