/* filename:    mcmcdd.g
** description: Computes numerical approximations to the data density
**              based on MCMC output
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

/******************************************************************
**         Load Parameter Draws from MH Output
*/
#include pathspec.g;
#include dsgespec.g;

mhrun = "1";

lpath  = parapath $+ "\\mhrun" $+ mhrun;
lstat  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "st";
lpara  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pa";

open fhpara = ^lpara for read;
open fhstat = ^lstat for read;

nburn   = 1001;
nblock  = 50000;
densfac = 1200;
hmax    = 2;

pararow = readr(fhpara,nburn);
npara   = sumc(para_maskinv[.,mspec+1]);
pararow = seekr(fhpara,nburn);
statrow = seekr(fhstat,nburn);

/* 
** Method:  modified harmonic mean
*/

p = seqa(0.1,0.1,9); 
pcrit = cdfchii(p,ones(rows(p),1)*npara);

/* Part 1: Compute posterior moments for parameters
*/

eofloop = 0;
ndraws  = 0;
sumtheta  = zeros(npara,1);
sumtheta2 = zeros(npara,npara);

do until eofloop; 

   thetasim = readr( fhpara, nblock );
   thetasim = delif(thetasim',para_mask[.,mspec+1])';

   sumtheta  = sumtheta + sumc(thetasim);
   sumtheta2 = sumtheta2 + thetasim'*thetasim;
   ndraws  = ndraws + rows(thetasim);

   eofloop = eof(fhpara);

endo;

thetahat      = sumtheta / ndraws;
thetasig      = sumtheta2 / ndraws - thetahat*thetahat';
thetasiginv   = inv(thetasig);
thetasiglndet = ln(det(thetasig));


/* Part 2:  Compute Density estimate
*/

pararow = seekr(fhpara,nburn);
statrow = seekr(fhstat,nburn);

eofloop = 0;
ndraws  = 0;
suminvlike = zeros(rows(p),1);
laginvlike = zeros(hmax,rows(p));
gaminvlike = zeros(hmax,rows(p));

do until eofloop;

   statsim  = readr( fhstat, nblock);
   thetasim = readr( fhpara, nblock);
   thetasim = delif(thetasim',para_mask[.,mspec+1])';

   i = 1;
   do until i > rows(thetasim);
    
      lnftheta = -ln(p) - 0.5*npara*ln(2*pi) - 0.5*thetasiglndet
                 -0.5*(thetasim[i,.] - thetahat')*thetasiginv*(thetasim[i,.]' - thetahat);
      indtheta = ((thetasim[i,.] - thetahat')*thetasiginv*(thetasim[i,.]' - thetahat) .< pcrit );
      invlike  = exp(lnftheta - statsim[i,1] + densfac).*indtheta;
       
      laginvlike = invlike' | laginvlike[1:hmax-1,.];
      gaminvlike = gaminvlike + laginvlike.*invlike';

      suminvlike = suminvlike + invlike;
      i = i+1;
   endo;

   ndraws = ndraws + rows(thetasim);
   locate 1,1;
   ndraws;
   eofloop = eof(fhpara);

endo;
closeall fhpara, fhstat;

meaninvlike = suminvlike/ndraws;
gaminvlike = gaminvlike/ndraws - (meaninvlike^2)';

/* Compute Standard Errors
*/
suminvlikeerror = gaminvlike[1,.];
j = 2;
do until j > hmax;
   suminvlikeerror = suminvlikeerror + 2*gaminvlike[j,.]*(1- (j-1)/hmax);
   j = j+1;
endo;
suminvlikeerror = 100*sqrt(suminvlikeerror/ndraws)./meaninvlike' ;

pym3        = densfac-ln(meaninvlike);

"Model:  " $lmodel;
"Prior:  " $lprior;
"MhRun:  " $mhrun;
"Dataset:" $dataselstr;
"======================";
"";

"Marginal Data Density" pym3;
"Average Across p     " meanc(pym3);
"Simulation Error [Percent]" suminvlikeerror';

end;


