/* filename:    filtdsge.g
** description: The program filters out technology and labor supply shocks 
** created:     01/24/06
*/

new;
closeall;
library pgraph, user;
cls;
#include pathspec.g;
#include dsgespec.g;
/******************************************************** 
** Import data on output and hours: series (nobs,2)
** observations from 1959:I to 2001:IV
*/
#include loaddata.g

nobs  = rows(series);       /* number of observations          */
YY    = series[1:nobs,.];   /* Note that YY is defined defined over the entire series */
T0=4; 

/* Open files with Parameter Draws
*/
mhrun  = "1";

lpara = parapath $+ "\\mhrun" $+ mhrun $+ "\\" $+ lmodel  $+ lprior $+ dataselstr $+ subTstr $+"pa";
open fhpara = ^lpara for read;

/* Define some Constants
*/
nsim   = 100;      /* block size */
skip   = 1;          /* skip = 1: use only first draw from block */
nburn  = 1000;

/* Initialize Output files
*/
opath = resupath $+ "\\mhrun" $+ mhrun $+ "\\";
chkdir=chk_dir(opath); 
oat = opath $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "at";
obt = opath $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "bt";

create fhat = ^oat with AT, nobs-T0, 8;
create fhbt = ^obt with BT, nobs-T0, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 0;
pararow = seekr(fhpara,nburn);

do until eofloop; 

   /* Read blocks of size nsim
   */
   parasim = readr( fhpara, nsim );

   nread = rows(parasim);
   
   if skip == 0;   
      Atsim = zeros(nread,nobs-T0);
      btsim = zeros(nread,nobs-T0);
    else;
      Atsim = zeros(1,nobs-T0);
      btsim = zeros(1,nobs-T0);
    endif;

   j = 1;
   do until j > rows(parasim);

      /* For each element of the block, conduct the filtering
      */   

     statesim = dsgefilt( parasim[j,.]', mspec, YY);
 
     if (mspec==0) or (mspec==2);  
     Atsim[j,.] = statesim[T0+1:nobs,2]';
     Btsim[j,.] = statesim[T0+1:nobs,1]';
     
     else;
     Atsim[j,.] = statesim[T0+1:nobs,1]';
     Btsim[j,.] = statesim[T0+1:nobs,2]';      
     
     endif; 

     "Iteration  " j "of" nsim;

     if skip == 1;
        j = rows(parasim)+2;
     else;
        j = j+1;
     endif;
  endo;

  /* Write Output
  */
  wr = writer(fhat,Atsim);
  wr = writer(fhbt,Btsim);

  loopct = loopct + 1;
  "Loop     " loopct;
  eofloop = eof(fhpara);

endo;

closeall fhat, fhbt, fhpara;

#include dsgemod.src;
#include dsgefilt.src;