/***************************************************************/
/*                                                             */
/*            Convert Impulse Response Functions               */
/*                                                             */
/*                       Frank Schorfheide                     */
/*                                                             */
/***************************************************************/

/* Filename:  irfconv.g
** created:   09/27/01
*/

new;
closeall;
library pgraph;
cls;

#include pathspec.g;
#include dsgespec.g;

lmodel = "m2112";

mhrun  = "1";

nread  = 1000;

modind = 1;
do until modind > rows(lmodel);
   
   mhind = 1;
   do until mhind > rows(mhrun);
     
         lpath  = resupath $+ "\\mhrun" $+ mhrun[mhind] $+ "\\";
         opath  = resupath $+ "\\mhrun" $+ mhrun[mhind] $+ "\\";

         lpath  = lpath $+ lmodel[modind];
         opath  = opath $+ lmodel[modind];

         
         "Model     " $lmodel[modind];
         "MhRun     " $mhrun[mhind];
         "=======================================";
         pause(1e-5);
         
         gosub transform;
         
      mhind = mhind + 1;
   endo;
   modind = modind+1;
endo;

end;

/*****************************************************************
**  S U B R O U T I N E
*/
transform:

/* Open Input Files
*/
lir11 = lpath $+ "r11";
lir12 = lpath $+ "r12";

lir21 = lpath $+ "r21";
lir22 = lpath $+ "r22";

lir31 = lpath $+ "r31";
lir32 = lpath $+ "r32";

open fhir11 = ^lir11 for read;
open fhir12 = ^lir12 for read;

open fhir21 = ^lir21 for read;
open fhir22 = ^lir22 for read;

open fhir31 = ^lir31 for read;
open fhir32 = ^lir32 for read;

irfrow = readr(fhir11,1);
nirf   = cols(irfrow);
irfrow = seekr(fhir11,1);

/* Initialize output files
*/
oic11 = opath $+ "c11";
oic12 = opath $+ "c12";

oic21 = opath $+ "c21";
oic22 = opath $+ "c22";

oic31 = opath $+ "c31";
oic32 = opath $+ "c32";

create fhic11 = ^oic11 with IR11, nirf, 8;
create fhic12 = ^oic12 with IR12, nirf, 8;

create fhic21 = ^oic21 with IR21, nirf, 8;
create fhic22 = ^oic22 with IR22, nirf, 8;

create fhic31 = ^oic31 with IR31, nirf, 8;
create fhic32 = ^oic32 with IR32, nirf, 8;

/* Convert Impulse Response Functions
*/

eofloop = 0;
ndraws  = 0;
loopct  = 0;

do until eofloop; 

   irf11sim = readr( fhir11, nread);
   irf12sim = readr( fhir12, nread);

   irf21sim = readr( fhir21, nread);
   irf22sim = readr( fhir22, nread);
 
   irf31sim = readr( fhir31, nread);
   irf32sim = readr( fhir32, nread);
   
   mult1 = 2*((irf11sim[.,1] .GE 0) -0.5);
   mult2 = 2*((irf22sim[.,1] .GE 0) -0.5);


/* Normalization:
** response of output to shock 1 > 0
**                 hour to shock 2 > 0
**        
*/

   irf11sim = irf11sim.*mult1;
   irf21sim = irf21sim.*mult1;
   irf31sim = irf31sim.*mult1; 

   irf12sim = irf12sim.*mult2;
   irf22sim = irf22sim.*mult2;
   irf32sim = irf32sim.*mult2;   

   wr = writer(fhic11,irf11sim);
   wr = writer(fhic12,irf12sim);
 
   wr = writer(fhic21,irf21sim);
   wr = writer(fhic22,irf22sim);

   wr = writer(fhic31,irf31sim);
   wr = writer(fhic32,irf32sim);

   loopct = loopct + 1;
   
   "Iteration" loopct;

   eofloop = eof(fhir11); 

endo;

closeall fhir11, fhir12,
         fhir21, fhir22, 
         fhir31, fhir32; 

return;
