/* filename:    dsgeprio1.g
** description: The program generates draws from the prior of the 
**              dsge model
** created:     12/14/01
** modified:    9/19/04
*/

library pgraph, user;
cls;

block_n    = 10;
block_size = 1000;
mhrun      = "0";   /* NOTE: we use mhrun = 0 to denote prior */

#include pathspec.g;
#include dsgespec.g

priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pri.out";
load path=^priopath prior[npara,3] = ^priorfile;

pmean  = prior[.,1];
pstdd  = prior[.,2];
pshape = prior[.,3];

pmean  = pmean.*para_maskinv[.,mspec+1] + para_fix[.,mspec+1].*para_mask[.,mspec+1];
pshape = pshape.*para_maskinv[.,mspec+1];

/* Initialize Output files
*/
opath = parapath $+ "\\mhrun" $+ mhrun;
chkdir=chk_dir(opath);
opara =opath $+ "\\"  $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pa";
create fhpara=^opara with PARAM, npara, 8;

/* Loop to generate parameter draws
*/
block_it = 1;
para_acpt = 0;

do until block_it > block_n;

   parasim = zeros(block_size,npara);
   i = 1; 
   do until i > npara;
   
      if pshape[i] == 1; /* BETA Prior */
         a = (1-pmean[i])*pmean[i]^2/pstdd[i]^2 - pmean[i];
         b = a*(1/pmean[i] - 1);
         parasim[.,i] = rndbeta(block_size,1,a,b);
      
      elseif pshape[i] == 2; /* GAMMA PRIOR */
         b = pstdd[i]^2/pmean[i];
         a = pmean[i]/b;
         parasim[.,i] = b*rndgam(block_size,1,a);
      
      elseif pshape[i] == 3; /* GAUSSIAN PRIOR */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = a + b*rndn(block_size,1);

      elseif pshape[i] == 4; /* INVGAMMA PRIOR */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = sqrt( b*a^2/sumc( (rndn(b,block_size))^2 ) );

      elseif pshape[i] == 5; /* uniform prior */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = a + (b-a)*rndu(block_size,1);

      elseif pshape[i] == 0; /* No Prior - Fixed */
         a = pmean[i];
         parasim[.,i] = a*ones(block_size,1);

      endif;

      i = i+1;
   endo; 
         
   
   "Processing Block" block_it;
   
   wr = writer(fhpara,parasim);
   block_it = block_it + 1;

endo;

closeall fhpara;
