dlibrary -a PszTgSen,PsdTgSen;

/* Model Specification
**
** 0 = stationary hour
** 1 = nonstationary hour 
** 2 = stationary hour with adjustment cost 
** 3 = nonstationary hour with adjustment cost 
*/
mspec  = 2;
lmodel = "m" $+ ftos(mspec,"%*.*lf",1,0) ;
mprior = 1;
lprior = ftos(mprior,"%*.*lf",1,0);

/* Selected data set
** 1 = BLS hour 
** 2 = CEV(2003) data : DRI-WEFA
** 3 = Haver 
** i > 4 = simulated data set i-4
*/
datasel = 1;

dataselstr = ftos(datasel,"%*.*lf",1,0);

/* Data Specification in Predictive Check 
** 1=hours in level 
** 2=hours in difference 
*/
dataspec=1;  

/*
** para_mask is npara * (number of model specifications)
**   to fix a parameter under specification mspec, set the corresponding
**   element of para_mask to 1
** para_fix is npara * (number of model specifications)
**   and contains the values for the fixed parameters (all other entries
**   are irrelevant
*/
nvar   = 2;
if (mspec==0) or (mspec==1); 
    npara  = 10;
else;
    npara=11;
endif;
ngrid  = 100;  /* number of grid points for density plots */

para_mask    = zeros(npara,4);  para_mask[8,2] = 1; para_mask[8,4]=1; 
para_maskinv = ones(npara,4) - para_mask; 
para_fix     = zeros(npara,4);  para_fix[8,2] = 1; para_fix[8,4]=1; 

/* Use subT to select subsample
*/
if datasel < 4;

   subT = 2;   
   subTstr = ftos(subT,"%*.*lf",1,0);
   
else;

   Tsimtotal = 10000;
   Tsimburn  = 1000;
   Tsim = 200 | 500 | 1000;
   subT = 1;   
   subTstr = ftos(subT,"%*.*lf",1,0);
     
endif;

/* Impulse responses / omega grid
*/
nirf   = 40;           /* number of impulse responses */
nomega = 50;           /* number of grid points for spectrum */
omegagr= seqa(0.0625, 0.33*pi/(nomega-1), nomega);
