/* filename:    filtplot.g
** description: Compute posterior mean 
**              and plot filtered shock processes
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;
#include pathspec.g;
#include dsgespec.g;

/******************************************************************
**         Load Parameter Draws from MH Output
*/

/* first model */ 
lmodel = "m2";
lprior = "1";
mhrun  = "1";
lpath  = resupath $+ "\\mhrun" $+ mhrun $+ "\\";

oat  = lpath $+ lmodel $+ lprior $+  dataselstr $+ subTstr $+ "at" ;
obt  = lpath $+ lmodel $+ lprior $+  dataselstr $+ subTstr $+ "bt" ;

open fhat = ^oat for read;
open fhbt = ^obt for read;

drawrow = readr( fhat, 1); 
drawdim = cols(drawrow);
drawrow = seekr( fhat, 1);

/* Part 1: Compute the mean of x(i)
*/
atmean0   = zeros(1,drawdim);
btmean0  = zeros(1,drawdim);

eofloop = 0;
nblock  = 5;
ndraws  = 0;

do until eofloop; 

   atblock = readr( fhat, nblock );
   btblock = readr( fhbt, nblock );
 
   atmean0  = atmean0 + sumc(atblock)';
   btmean0  = btmean0 + sumc(btblock)';
 
   ndraws = ndraws + rows(atblock);

   locate 1,1;
   "Part 1";
   "Draws " ndraws;

   eofloop = eof(fhat);

endo;

atmean0  = atmean0'/ndraws;
btmean0  = btmean0'/ndraws;

nobs  = rows(atmean0);
ti       = seqa(1960,0.25,nobs);

closeall fhat, fhbt; 

/*second model */ 
lmodel = "m2";
lprior = "1";
mhrun  = "1";
lpath  = resupath $+ "\\mhrun" $+ mhrun $+ "\\";

oat= lpath $+ lmodel $+ lprior $+  dataselstr $+ subTstr $+ "at" ;
obt= lpath $+ lmodel $+ lprior $+  dataselstr $+ subTstr $+ "bt" ;

open fhat= ^oat for read;
open fhbt= ^obt for read;

drawrow = readr( fhat, 1); 
drawdim = cols(drawrow);
drawrow = seekr( fhat, 1);

/* Part 1: Compute the mean of x(i)
*/
atmean1   = zeros(1,drawdim);
btmean1  = zeros(1,drawdim);

eofloop = 0;
nblock  = 5;
ndraws  = 0;

do until eofloop; 

   atblock = readr( fhat, nblock );
   btblock = readr( fhbt, nblock );
 
   atmean1  = atmean1 + sumc(atblock)';
   btmean1  = btmean1 + sumc(btblock)';
 
   ndraws = ndraws + rows(atblock);

   locate 1,1;
   "Part 1";
   "Draws " ndraws;

   eofloop = eof(fhat);

endo;

atmean1  = atmean1'/ndraws;
btmean1  = btmean1'/ndraws;

nobs  = rows(atmean1);
ti       = seqa(1960,0.25,nobs);


/* Plot Impulse Response Functions
*/

graphset;

   begwind;
   margin(0,0,0.2,0.2);
   window(2,1,0);
   fonts("microb");

   _ptitlht = 0.3;
   _paxht = 0.3;
   _pnumht = 0.25;
   _pcolor = {2 4} ;
   _pltype = {6 1};
   _protate = 0; 
   _plctrl = 0;
   _plwidth = 5.5;
   _psymsiz = 5;
   _plegstr = "Model A]0[\000Model A]1[";
   xlabel("Year");
   xtics(1960,2002,1,1);
 

ylow  =0.01*round(100*minc(minc(atmean0~atmean1)));   ylow=ylow-0.01*round(abs(ylow));
yhigh =0.01*round(100*maxc(maxc(atmean0~atmean1)));  yhigh=yhigh+0.01*round(abs(yhigh)); 
ytics(ylow,yhigh,0.05,1); 

_pline = ( 1 ~ 1 ~ 1958.25 ~ ylow ~ 1958.25 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1961 ~ ylow ~ 1961 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1970.75 ~ ylow ~ 1970.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1975 ~ ylow ~ 1975 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1980.5 ~ ylow ~ 1980.5 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1982.75 ~ ylow ~ 1982.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1991 ~ ylow ~ 1991 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 2001.75 ~ ylow ~ 2001.75 ~ yhigh ~ 4 ~ 1 )|  
            ( 1 ~ 6 ~ 1957.5 ~ ylow ~ 1957.5 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 6 ~ 1960.25 ~ ylow ~ 1960.25 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1969.75 ~ ylow ~ 1969.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1973.75 ~ ylow ~ 1973.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1980 ~ ylow ~ 1980 ~ yhigh ~ 4~ 1 ) |
            ( 1 ~ 6 ~ 1981.5 ~ ylow ~ 1981.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1990.5 ~ ylow ~ 1990.5 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 6 ~ 2001 ~ ylow ~ 2001 ~ yhigh ~ 4 ~ 1) ;
            
  title("Technology : ln a(t)");
   _plegctl = {2 5 7 1.5};
   xy(ti,atmean0~atmean1);
   nextwind;

   
 
ylow=0.01*round(100*minc(minc(btmean0~btmean1))); ylow=ylow-0.01*abs(ylow); 
yhigh=0.01*round(100*maxc(maxc(btmean0~btmean1))); yhigh=yhigh+0.01*abs(yhigh); 
ytics(ylow,yhigh,0.02,1); 


   _pline = ( 1 ~ 1 ~ 1958.25 ~ ylow ~ 1958.25 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1961 ~ ylow ~ 1961 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1970.75 ~ ylow ~ 1970.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1975 ~ ylow ~ 1975 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1980.5 ~ ylow ~ 1980.5 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1982.75 ~ ylow ~ 1982.75 ~ yhigh ~ 4 ~ 1 )|
            ( 1 ~ 1 ~ 1991 ~ ylow ~ 1991 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 1 ~ 2001.75 ~ ylow ~ 2001.75 ~ yhigh ~ 4 ~1 )|  
            ( 1 ~ 6 ~ 1957.5 ~ ylow ~ 1957.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1960.25 ~ ylow ~ 1960.25 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1969.75 ~ ylow ~ 1969.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1973.75 ~ ylow ~ 1973.75 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1980 ~ ylow ~ 1980 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1981.5 ~ ylow ~ 1981.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 1990.5 ~ ylow ~ 1990.5 ~ yhigh ~ 4 ~ 1 ) |
            ( 1 ~ 6 ~ 2001 ~ ylow ~ 2001 ~ yhigh ~ 4 ~ 1) ;
   title("Labor supply : ln b(t)");
   _plegctl = { };
   xy(ti,btmean0~btmean1);
   endwind;



