/******************************************************************/
/*                                                                */
/*              Plot the impulse response functions               */
/*                                                                */
/*                        Frank Schorfheide                       */ 
/*                      modified by Taeyoung Doh                  */
/*                                                                */
/******************************************************************/

/* filename: irfplot2.g
** created:  09/27/01
*/
new;
library pgraph;
cls;
#include pathspec.g;
#include dsgespec.g; 


ti    = seqa(1,1,nirf);

/* Read Posterior Simulation Summary Files
*/

/* DSGE stationary model
*/
lmodel = "m2";
lprior = "1";
mhrun  = "1";

lirf   = "11";
osummary       = resupath $+"\\mhrun" $+ mhrun $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "c" $+ lirf $+"s";
open fhsummary = ^osummary for read;
m0ir11 = readr(fhsummary,1)';
seekr(fhsummary,3+nirf);
m0ir11band = readr(fhsummary,2)';
closeall fhsummary;

lirf   = "12";
osummary       = resupath $+ "\\mhrun" $+ mhrun $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "c" $+ lirf $+"s";
open fhsummary = ^osummary for read;
m0ir12 = readr(fhsummary,1)';
seekr(fhsummary,3+nirf);
m0ir12band = readr(fhsummary,2)';
closeall fhsummary;


lirf   = "31";
osummary       = resupath $+"\\mhrun" $+ mhrun $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "c" $+ lirf $+"s";
open fhsummary = ^osummary for read;
m0ir21 = readr(fhsummary,1)';
seekr(fhsummary,3+nirf);
m0ir21band = readr(fhsummary,2)';
closeall fhsummary;

lirf   = "32";
osummary       = resupath $+"\\mhrun" $+ mhrun $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "c" $+ lirf $+"s";
open fhsummary = ^osummary for read;
m0ir22 = readr(fhsummary,1)';
seekr(fhsummary,3+nirf);
m0ir22band = readr(fhsummary,2)';
closeall fhsummary; 


/* DSGE non-stationary model 
*/
lmodel = "m2";
lprior = "1";
mhrun  = "1";

lirf   = "11";
osummary   = resupath $+ "\\mhrun" $+ mhrun $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "c" $+ lirf $+"s";
open fhsummary = ^osummary for read;
m1ir11 = readr(fhsummary,1)';
seekr(fhsummary,3+nirf);
m1ir11band = readr(fhsummary,2)';
closeall fhsummary;

lirf   = "12";
osummary   = resupath $+ "\\mhrun" $+ mhrun $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "c" $+ lirf $+"s";
open fhsummary = ^osummary for read;
m1ir12 = readr(fhsummary,1)';
seekr(fhsummary,3+nirf);
m1ir12band = readr(fhsummary,2)';
closeall fhsummary;

lirf   = "31";
osummary   = resupath $+ "\\mhrun" $+ mhrun $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "c" $+ lirf $+"s";
open fhsummary = ^osummary for read;
m1ir21 = readr(fhsummary,1)';
seekr(fhsummary,3+nirf);
m1ir21band = readr(fhsummary,2)';
closeall fhsummary;

lirf   = "32";
osummary  = resupath $+ "\\mhrun" $+ mhrun $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "c" $+ lirf $+"s";
open fhsummary = ^osummary for read;
m1ir22 = readr(fhsummary,1)';
seekr(fhsummary,3+nirf);
m1ir22band = readr(fhsummary,2)';
closeall fhsummary;


/* Plot Impulse Response Functions
*/
pltirf:   
   graphset;
   begwind;
   /*
   margin(0,0,0.2,0.2);
   window(2,2,0);
   */
   makewind(2.5, 3, 0.5, 3.0, 0);
   makewind(2.5, 3, 3.5, 3.0, 0);
   makewind(2.5, 3, 0.5, 0, 0);
   makewind(2.5, 3, 3.5, 0, 0);
   _pagesiz = {6,6};

   fonts("microb");
   _ptitlht = 0.4;
   _paxht  = 0.4;
   _pnumht = 0.4;
   _pcolor = {1 1 4 4} ;
   _pltype = {6 6 3 3};
   _protate = 0; 
   _plctrl = 0;
   _plwidth = 5.5;
   _psymsiz = 5;
  _plegstr = "Model A]0[ (high)\000Model A]0[ (low)\000Model A]1[ (high)\000Model A]1[ (low)";
   
   ylabel("Hours (%)");   
   title("Technology Shock");
   ytics(0,0.5,0.25,1); 
   _plegctl = {2 4 2.7 4.2};
   xy(ti,m0ir11band~m1ir11band);
   nextwind;

   title("Labor Supply Shock");
   ytics(0,1.5,0.5,1);
   _plegctl = { };
   xy(ti,m0ir12band~m1ir12band);
   nextwind;

   ytics(0,1.5,0.5,1);
   ylabel("Output (%)");   
   title("Technology Shock");
   _plegctl = { };
   xy(ti,m0ir21band~m1ir21band );
   nextwind;

   title("Labor Supply Shock");
   _plegctl = { };
   xy(ti,m0ir22band~m1ir22band );
 
 
  endwind;
