/******************************************************************************
* fig_draw.g
* plots bivariate scatters
* 
*
* upadted:  03/07/2006
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

#include pathspec.g
#include dsgespec.g
#include dsgemod.src;

/********************************************************************
* Load Parameter Draws -- Posterior
********************************************************************/

dataselstr = "1";
mspec      = 2;
npara      = 11;
mprior     = 1;
_verbose   = 0;

ndata          = rows(dataselstr);

priormean      = zeros(ndata,npara); 
posteriormean  = zeros(ndata,npara); 

priordraws      = zeros(1,npara);
priorblocks     = zeros(ndata,1);

posteriordraws  = zeros(1,npara);
posteriorblocks = zeros(ndata,1);

ind=1;
do until ind > ndata; 
/** BEGIN DATA LOOP **/

mhrun  = "1";
nburn  = 11;
nblock = 90000;    /* block size */
nuse   = 20;     /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
lpara  = lpath $+"\\" $+ lmodel $+ lprior $+ dataselstr[ind] $+ subTstr $+ "pa";

open fhpara = ^lpara for read;

drawrow = seekr(fhpara,nburn);
drawblock = readr(fhpara,1);

loopct = 1;
eofloop = 0;

do until eofloop;

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   parasim = delif(parasim,indseq);

   if ismiss(parasim) == 0;
 
      drawblock = drawblock | parasim;
      
   endif;
   
   if _verbose;
      "";
      "Draws Processed " ftos(rows(parasim),"%*.*lf",6,0);
        "";
   endif;

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;

closeall fhpara;

posteriordraws = posteriordraws | drawblock;
posteriorblocks[ind] = rows(drawblock);

/********************************************************
* load Posterior Mean Estimates
********************************************************/
lpara  = lpath $+"\\" $+ lmodel $+ lprior $+ dataselstr[ind] $+ subTstr $+ "pas";

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1);
closeall fhpara;

posteriormean[ind,.] = para;

/********************************************************************
* Load Parameter Draws -- Prior
********************************************************************/
mhrun  = "0";
nburn  = 1;
nblock = 100000;    /* block size */
nuse   = 20;     /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
lpara  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr[ind] $+ subTstr $+ "pa";

open fhpara = ^lpara for read;

drawrow = seekr(fhpara,nburn);
drawblock = readr(fhpara,1);


loopct = 1;
eofloop = 0;

do until eofloop;

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   parasim = delif(parasim,indseq);

   if ismiss(parasim) == 0;
 
      drawblock = drawblock | parasim;
      
   endif;
   
   if _verbose;
      "";
      "Draws Processed " ftos(rows(parasim),"%*.*lf",6,0);
        "";
   endif;

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;

closeall fhpara;

priordraws = priordraws | drawblock;
priorblocks[ind] = rows(drawblock);

/********************************************************
* load Prior Mean Estimates
********************************************************/
lpara  = lpath $+"\\" $+ lmodel $+ lprior $+ dataselstr[ind] $+ subTstr $+ "pas";

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1);
closeall fhpara;

priormean[ind,.] = para;

/** END DATA LOOP **/
ind=ind+1;
endo;

priordraws     = priordraws[2:rows(priordraws),.];
posteriordraws = posteriordraws[2:rows(posteriordraws),.];
priorblocks     = 0 | cumsumc(priorblocks);
posteriorblocks = 0 | cumsumc(posteriorblocks);

/********************************************************************
* Create Plots
********************************************************************/

para_names = "alp" $| 
             "bet" $|
             "gam" $|
             "del" $|
             "nu" $|
             "ln A0" $|
             "ln B0" $|
             "RHO]b[" $|
             "sigA" $|
             "sgB" $| 
             "PHI" ;

para_indvec = (8 ~ 11 ) | 
              (8 ~ 11 ) |
              (8 ~ 11 );              
 
para_tics =  (0.5 ~ 1 ~ 0.25)| 
             (0 ~ 60 ~ 20); 

graphset;
begwind;
/*
margin(0.2,0.2,0.2,0.2); 
window(ndata,2,0);
*/

makewind(2.5, 3, 0.5, 3.0, 0);
makewind(2.5, 3, 3.5, 3.0, 0);
makewind(2.5, 2.5, 0.5, 0, 0);
makewind(2.5, 2.5, 3.5, 0, 0);
_pagesiz = {6,6};

fonts("microb");
_ptitlht = 0.6;
_paxht = 0.4;
_pnumht = 0.4;
_pcolor = {1 4 4};
_pltype = {6 2 2};
_protate = 0;
_plctrl = -1;
_plwidth = 5.5;
_psymsiz = 2;

_pypmax = 1;

i = 1;
do until i > ndata; 
     
  _pline = ( 1 ~ 6 ~ para_tics[1,1] ~ posteriormean[i,para_indvec[i,2]] ~ para_tics[1,2] ~ posteriormean[i,para_indvec[i,2]] ~ 4 ~ 1) |
           ( 1 ~ 6 ~ posteriormean[i,para_indvec[i,1]] ~ para_tics[2,1] ~ posteriormean[i,para_indvec[i,1]] ~ para_tics[2,2] ~ 4 ~ 1) |
           ( 1 ~ 3 ~ para_tics[1,1] ~ priormean[i,para_indvec[i,2]] ~ para_tics[1,2] ~ priormean[i,para_indvec[i,2]] ~ 4 ~ 1) |
           ( 1 ~ 3 ~ priormean[i,para_indvec[i,1]] ~ para_tics[2,1] ~ priormean[i,para_indvec[i,1]] ~ para_tics[2,2] ~ 4 ~ 1) ;

  xtics(para_tics[1,1],para_tics[1,2],para_tics[1,3],1);
  ytics(para_tics[2,1],para_tics[2,2],para_tics[2,3],1);

  xlabel(para_names[para_indvec[i,1]]);
  ylabel(para_names[para_indvec[i,2]]);

  title("Prior Draws");
  xy(priordraws[priorblocks[i]+1:priorblocks[i+1],para_indvec[i,1]],
     priordraws[priorblocks[i]+1:priorblocks[i+1],para_indvec[i,2]]);
  nextwind;

  title("Posterior Draws");
  xy(posteriordraws[posteriorblocks[i]+1:posteriorblocks[i+1],para_indvec[i,1]],
     posteriordraws[posteriorblocks[i]+1:posteriorblocks[i+1],para_indvec[i,2]]);
  nextwind; 

  i=i+1;
endo;
endwind;

         
end;
