/******************************************************************************
* dsgepchk.G
* plots draws from posterior predictive distribution and the observed moment
* 
* upadted:  03/09/2006
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include dsgespec.g;
#include pathspec.g;

/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/

drawmat = {};
drawmatpr={}; 

mhrun   = "0"$|"1";
mhind=1; 
do until mhind>2; 

nburn   = 1;    /* initial draws to be discarded */
nuse    = 1;

lpath  = resupath $+ "\\mhrun" $+ mhrun[mhind];

ldraw  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "acsamp";
ldata  = lpath $+ "\\sampdata";

open fhdraw = ^ldraw for read;
open fhdata = ^ldata for read;

drawrow = readr(fhdraw, nburn);
drawdim = cols(drawrow);
drawrow = seekr(fhdraw, nburn);

/* select two moments from the following list
** 1-2   MU_DY MU_H
** 3-6   V_DY C0_DYH  C0_HDY  V_H 
** 7-10   C1_DY C1_DYH C1_HDY C1_H
*/

momsel = (3 ~ 7) |
         (6 ~ 10) |
         (5 ~ 8 )  ;

if dataspec==1; 
  draw_tics = -ones(10,1) ~ ones(10,1) ~ 0.5*ones(10,1);
  draw_tics[3,.] = ( 0 ~ 0.1 ~ 0.05);
  draw_tics[5,.] = ( -0.5 ~ 0.5 ~ 0.25);
  draw_tics[6,.] = ( 0 ~ 0.1 ~ 0.05);
  draw_tics[7,.] = ( -0.25 ~ 0.75~ 0.25);
  draw_tics[8,.] = (-0.5 ~ 1 ~ 0.25);
  draw_tics[10,.] = ( 0.5 ~ 1 ~ 0.25);

  draw_names = "Mean (D_Y)" $|
             "Mean (H)" $|
             "Std (D_Y)" $|
             "Corr (D_Y, H)" $|
             "Corr (H, D_Y)" $|             
             "Std (H)" $|
             "Corr (D_Y, D_Y(-1))" $|
             "Corr (D_Y, H(-1))" $|
             "Corr (H, D_Y(-1))" $|
             "Corr (H, H(-1))" ;
else; 
  draw_tics = -ones(10,1) ~ ones(10,1) ~ 0.5*ones(10,1);
  draw_tics[3,.] = (  0 ~ 0.1 ~ 0.05);
  draw_tics[5,.] = ( 0.25 ~ 1 ~ 0.25);
  draw_tics[6,.] = ( 0 ~ 0.1 ~ 0.02);
  draw_tics[7,.] = ( 0~ 0.75~ 0.25);
  draw_tics[8,.] = (-0.5 ~ 1 ~ 0.25);
  draw_tics[10,.] = ( -0.25 ~ 1 ~ 0.25);

  draw_names = "Mean (D_Y)" $|
             "Mean (D_H)" $|
             "Std (D_Y)" $|
             "Corr (D_Y,D_H)" $|
             "Corr (D_H, D_Y)" $|             
             "Std (D_H)" $|
             "Corr (D_Y, D_Y(-1))" $|
             "Corr (D_Y, D_H(-1))" $|
             "Corr (D_H, D_Y(-1))" $|
             "Corr (D_H, D_H(-1))" ;
endif;
     

/********************************************************************
* Part 1: load actual sample moments
********************************************************************/

if mhind==1; 
sampmomdata  = readr(fhdata,1);
endif; 


 /********************************************************************
* Part 2: load draws from posterior predictive distribution
********************************************************************/
eofloop = 0;
nblock  = 10;
ndraws  = 0;

do until eofloop;

   drawblock  = readr(fhdraw,nblock);
   nread   = rows(drawblock);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   drawblock = delif(drawblock,indseq);
   nread   = rows(drawblock);
   if mhind==1; 
   drawmatpr = drawmatpr | drawblock;
   else; 
   drawmat=drawmat | drawblock; 
   endif; 

   ndraws  = ndraws + nread;

   ".";;pause(1e-20);

   eofloop = eof(fhdraw);

endo;


closeall fhdraw, fhdata;
mhind=mhind+1;
endo; 

         

/********************************************************************
* Part 3: Plot the results
********************************************************************/

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {6 1 2}; 
_pcolor = 1;
_plwidth = 3.5;
_plctrl = -1;
begwind;
window(3,2,0);

i = 1;
do until i > 3;

   xlabel(draw_names[momsel[i,1]]);
   ylabel(draw_names[momsel[i,2]]);

  
  _pline = ( 1 ~ 6 ~ draw_tics[momsel[i,1],1] ~ sampmomdata[momsel[i,2]] ~ draw_tics[momsel[i,1],2] ~ sampmomdata[momsel[i,2]] ~ 4 ~ 1) |
              ( 1 ~ 6 ~ sampmomdata[momsel[i,1]] ~ draw_tics[momsel[i,2],1] ~ sampmomdata[momsel[i,1]] ~ draw_tics[momsel[i,2],2] ~ 4 ~ 1) ;

   xtics(draw_tics[momsel[i,1],1],draw_tics[momsel[i,1],2],draw_tics[momsel[i,1],3],1);
   ytics(draw_tics[momsel[i,2],1],draw_tics[momsel[i,2],2],draw_tics[momsel[i,2],3],1);

   xy(drawmatpr[.,momsel[i,1]],drawmatpr[.,momsel[i,2]]);
   nextwind;
    
   xy(drawmat[.,momsel[i,1]],drawmat[.,momsel[i,2]]); 
   nextwind;      

   i = i+1;

endo;

endwind;

end;
