/******************************************************************************
* dsgepchk.G
* plots draws from posterior predictive distribution and the observed moment
* 
* upadted:  03/09/2006
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include dsgespec.g;
#include pathspec.g;

/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/


model_id = "m2112" $|
           "m2112" $|
           "m2112" $|
           "m2112" ;

dataselstr = "1";

title_names = "Model M]0[" $|
              "Model M]1[" $|
              "Model A]0[" $|
              "Model A]1[" ;

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {6 1 2}; 
_pcolor = 1;
_plwidth = 3.5;
_plctrl = -1;
_psymsiz = 2;


begwind;
makewind(2.5, 3, 0.5, 3.0, 0);
makewind(2.5, 3, 3.5, 3.0, 0);
makewind(2.5, 3, 0.5, 0, 0);
makewind(2.5, 3, 3.5, 0, 0);
_pagesiz = {6,6};

model_ind=1; 
do until model_ind > rows(model_id);

/** BEGIN LOOP for MODELS **/ 

nburn   = 1;   
nuse    = 1;

lpath  = resupath $+ "\\mhrun1";
ldraw  = lpath    $+ "\\" $+ model_id[model_ind] $+ "acsamp";
ldata  = resupath $+ "\\mhrun0\\"  $+ dataselstr $+ "acsampdata";

open fhdraw = ^ldraw for read;
open fhdata = ^ldata for read;

drawrow = readr(fhdraw, nburn);
drawdim = cols(drawrow);
drawrow = seekr(fhdraw, nburn);

drawmat = {};

/* select two moments from the following list
** 1-2   MU_DY MU_H
** 3-6   V_DY C0_DYH  C0_HDY  V_H 
** 7-10  C1_DY C1_DYH C1_HDY C1_H
*/

momsel = (6 ~ 10); 
         
if dataspec==1; 
  draw_tics = -ones(10,1) ~ ones(10,1) ~ 0.5*ones(10,1);
  draw_tics[3,.] = ( 0 ~ 0.1 ~ 0.05);
  draw_tics[5,.] = ( -0.5 ~ 0.5 ~ 0.25);
  draw_tics[6,.] = ( 0 ~ 0.1 ~ 0.05);
  draw_tics[7,.] = ( -0.25 ~ 0.75~ 0.25);
  draw_tics[8,.] = (-0.5 ~ 1 ~ 0.25);
  draw_tics[10,.] = ( 0.8 ~ 1 ~ 0.1);

  draw_names = "Mean (D_Y)" $|
               "Mean (H)  " $|
               "Std (D_Y)" $|
               "Corr (D_Y, H)" $|
               "Corr (H, D_Y)" $|             
               "Standard Deviation" $|
               "Corr (D_Y, D_Y(-1))" $|
               "Corr (D_Y, H(-1))" $|
               "Corr (H, D_Y(-1))" $|
               "Autocorrelation" ;
else; 
  draw_tics = -ones(10,1) ~ ones(10,1) ~ 0.5*ones(10,1);
  draw_tics[3,.] = (  0 ~ 0.1 ~ 0.05);
  draw_tics[5,.] = ( 0.25 ~ 1 ~ 0.25);
  draw_tics[6,.] = ( 0 ~ 0.1 ~ 0.02);
  draw_tics[7,.] = ( 0~ 0.75~ 0.25);
  draw_tics[8,.] = (-0.5 ~ 1 ~ 0.25);
  draw_tics[10,.] = ( -0.25 ~ 1 ~ 0.25);

  draw_names = "Mean (D_Y)" $|
               "Mean (D_H)" $|
               "Std (D_Y)" $|
               "Corr (D_Y,D_H)" $|
               "Corr (D_H, D_Y)" $|             
               "Standard Deviation" $|
               "Corr (D_Y, D_Y(-1))" $|
               "Corr (D_Y, H(-1))" $|
               "Corr (D_H, D_Y(-1))" $|
               "Autocorrelation" ;
endif;
     

/********************************************************************
* Part 1: load actual sample moments
********************************************************************/

sampmomdata  = readr(fhdata,1);

 /********************************************************************
* Part 2: load draws from posterior predictive distribution
********************************************************************/
eofloop = 0;
nblock  = 10;
ndraws  = 0;

do until eofloop;

   drawblock  = readr(fhdraw,nblock);
   nread   = rows(drawblock);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   drawblock = delif(drawblock,indseq);
   nread   = rows(drawblock);
   drawmat=drawmat | drawblock; 

   ndraws  = ndraws + nread;

   ".";;pause(1e-20);

   eofloop = eof(fhdraw);

endo;

closeall fhdraw, fhdata;

/********************************************************************
* Part 3: Plot the results
********************************************************************/

i = 1;
do until i > rows(momsel);

   xlabel(draw_names[momsel[i,1]]);
   ylabel(draw_names[momsel[i,2]]);
   title(title_names[model_ind]);
  
  _pline = ( 1 ~ 6 ~ draw_tics[momsel[i,1],1] ~ sampmomdata[momsel[i,2]] ~ draw_tics[momsel[i,1],2] ~ sampmomdata[momsel[i,2]] ~ 4 ~ 1) |
           ( 1 ~ 6 ~ sampmomdata[momsel[i,1]] ~ draw_tics[momsel[i,2],1] ~ sampmomdata[momsel[i,1]] ~ draw_tics[momsel[i,2],2] ~ 4 ~ 1) ;

   xtics(draw_tics[momsel[i,1],1],draw_tics[momsel[i,1],2],draw_tics[momsel[i,1],3],1);
   ytics(draw_tics[momsel[i,2],1],draw_tics[momsel[i,2],2],draw_tics[momsel[i,2],3],1);

   xy(drawmat[.,momsel[i,1]],drawmat[.,momsel[i,2]]);
   nextwind;
   i = i+1;
   
endo;

/** END of MODEL LOOP **/
model_ind = model_ind+1;
endo; 


endwind;

end;
