/* filename:    paratab1.g
** description: generates a LaTeX Table  
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

/******************************************************************
**         Load Parameter Draws from MH Output
*/
#include pathspec.g;
#include dsgespec.g;

/* Specify input files
** filename: lmodel + lprior + dataselstr + subTstr + pas
*/
inputfile_str = "\\mhrun0\\m2112pas";

/* Specify row names for table
** Note: for models without adjustment costs comment out \\varphi
*/
row_name = "$\\alpha$     & $[0,1)$  & Beta  "
         $| "$\\beta$     & $\\RR^+$ & Gamma "
         $| "$\\gamma$    & $\\RR^+$ & Gamma "
         $| "$\\delta$    & $\\RR^+$ & Gamma "
         $| "$\\nu$       & $[0,1)$  & Beta  "
         $| "$\\lnA_0$    & $[0,1)$  & Beta  "
         $| "$\\lnB_0$    & $[0,1)$  & Beta  "
         $| "$\\rho_B$    & $[0,1)$  & Beta  "
         $| "$\\sigma_A$  & $\\RR^+$ & InvGamma "
         $| "$\\sigma_B$  & $\\RR^+$ & InvGamma "
         $| "$\\varphi$   & $\\RR^+$ & Gamma ";

        
             
inputfile_n = rows(inputfile_str);

post_mean = zeros(npara,inputfile_n);
post_ci   = zeros(npara,inputfile_n*2);


/* read the input files
*/
inputfile_ind = 1;

do until inputfile_ind > inputfile_n;

   lparas = parapath $+ inputfile_str[inputfile_ind];
   open fhparas = ^lparas for read;
   
   post_mean[.,inputfile_ind] = readr(fhparas,1)';
   drawrow = seekr(fhparas,npara+3);
   post_ci[.,2*(inputfile_ind-1)+1:2*inputfile_ind] = readr(fhparas,2)';

   inputfile_ind = inputfile_ind+1;
   
   closeall fhparas;
   
endo;

/* Create table
*/
para_ind = 1;
do until para_ind > npara;

   tablerow = row_name[para_ind];

   inputfile_ind = 1;
   do until inputfile_ind > inputfile_n;
      
      tablerow = tablerow $+ " & " $+ ftos(post_mean[para_ind,inputfile_ind],"%*.*lf",5,2)
                 $+ " & [" $+ ftos(post_ci[para_ind,2*(inputfile_ind-1)+1],"%*.*lf",5,2)
                 $+ ", "  $+ ftos(post_ci[para_ind,2*(inputfile_ind)],"%*.*lf",5,2) $+ "]";
      inputfile_ind = inputfile_ind+1;
      
   endo;
   tablerow  = tablerow  $+ "\\\\";
   
   $tablerow;
   
   para_ind = para_ind+1;

endo;
