   /********************************************************************
Proc            : ADF_LS
Authors         : Alan G. Isaac and David Rapach
Last revised    : 27 May 1996
Input           : y   -- (N x 1) vector, the time series of interest
                  p   -- scalar, maximum lag (max autocorr order)
                  prt -- string, prt="on" to print results
Output          : Prints augmented Dickey-Fuller t-stats
                  b   -- coefs from ADF regression
                  t   -- t-stats from ADF regression
                  obs -- scalar, rows(y)-(p+1)
Globals         : None
Notes           : Requires procs `varlags()' (provided)
References      : Russell Davidson and James G. MacKinnon,
                      _Estimation and Inference in Econometrics_
                      (New York: Oxford, 1993)
                      See references therein for original sources
********************************************************************/
 /******************************************************** 
** Import data
** series (nobs,2)
*/
#include loaddata.g
 
n  = rows(series);                     /* number of observations  */
y  = series[1:nobs,2];   
p=4; 

{b,t,obs}=adf_ls(y,p,"on"); 
proc(3)=adf_ls(y,p,prt);
    local labelx,addx,y_1,dy,dylags,temp,report,rhs,gap,oldcv,oldnv,
          T,b,e,SSE,sig2,covb,seb,ty,i;
    T=rows(y);                     @ number of observations @
    labelx="x(-1)";
    addx={};
    {y,y_1}=varlags(y,1);          @ generating lags @
    dy=y-y_1;                      @ differencing @
    if p /= 0;
      {dy,dylags}=varlags(dy,p);   @ generating lags @
      addx=addx~dylags;
      labelx=labelx|(0$+"dx(-"$+ftocv(seqa(1,1,p),1,0)$+")");
    endif;
    labelx=labelx|"constant"|"trend"|"RSS";
    T=rows(dy);                    @ # of usable obs @
    y_1=y_1[p+1:p+T];              @ usable lagged y @
    addx=y_1~addx;
    gosub regress(addx,miss(zeros(2,1),0));
      pop temp;
      report=temp;
    addx=addx~ones(T,1);           @ add constant to regressors @
    gosub regress(addx,miss(0,0)); @ add constant to regressors @
      pop temp;
      report=report~temp;
    addx=addx~seqa(-int(T/2),1,T); @ add trend to regressors @
    temp={};
    gosub regress(addx,temp);
      pop temp;
      report=report~temp;
    if prt $== "on";
      print;
      "ADF results for "$+ftocv(p,1,0)$+" lags";
"------------------------------------------------------------
                      ADF regression coefficients
                             (t-ratio)
                For model including the following:
                ----------------------------------
                No Constant  Constant    Trend
     RHS var    No Trend     No Trend    Constant
--------------------------------------------------";
    i=1;
    oldcv=__fmtcv;oldnv=__fmtnv;
    __fmtcv="*.*s"~12~8;__fmtnv="*.*lg"~12~3;
    temp=printfmt(labelx[1]~report[1,1|3|5],0~1~1~1);print;
    do while i<=p+3;
      temp=printfmt("  "~(0$+"("$+ftocv(report[i,2|4|6],1,2)$+")")~"  ",0);
      print;
      i=i+1;
      temp=printfmt(labelx[i]~report[i,1|3|5],0~1~1~1);print;
    endo;
"------------------------------------------------------------";
"Note: Final line is sum of squared residuals";
/* Could use these for Dickey Fuller (1981) Phi tests */
        __fmtcv=oldcv;__fmtnv=oldnv;
    endif;
retp(report[.,1|3|5],report[.,2|4|6],T); @ coef, t-stat, usable obs @
regress:
    pop gap;
    pop rhs;
    b=dy/rhs;                     @ OLS estimates @
    e=dy-rhs*b;                   @ resids @
    SSE=e'e;                      @ sum of squared resids @
    sig2=SSE/(T-cols(rhs));       @ e2 estimate @
    covb=sig2*invpd(rhs'rhs);     @ var-cov(b) @
    seb=sqrt(diag(covb));         @ std errs @
    return((b|gap|SSE)~((b./seb)|gap|miss(0,0)));
endp;

/**********************  PROC VARLAGS  *****************************
**   author: Alan G. Isaac
**   last update: 5 Dec 95      previous: 15 June 94
**   FORMAT
**          { x,xlags } = varlags(var,lags)
**   INPUT
**        var  - T x K matrix
**        lags - scalar, number of lags of var (a positive integer)
**   OUTPUT
**          x -     (T - lags) x K matrix, the last T-lags rows of var
**          xlags - (T - lags) x lags*cols(var) matrix,
**                  being the 1st through lags-th
**                  values of var corresponding to the values in x
**                  i.e, the appropriate rows of x(-1)~x(-2)~etc.
**   GLOBAL VARIABLES: none
**********************************************************************/
proc(2)=varlags(var,lags);
    local xlags;
    xlags = shiftr((ones(1,lags) .*. var)',seqa(1-lags,1,lags)
                                            .*. ones(cols(var),1),miss(0,0))';
    retp(trimr(var,lags,0),trimr(xlags,0,lags));
endp;

